package cn.com.infosec.crypto.agreement;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.params.AsymmetricKeyParameter;
import cn.com.infosec.crypto.params.DHParameters;
import cn.com.infosec.crypto.params.DHPrivateKeyParameters;
import cn.com.infosec.crypto.params.DHPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.math.BigInteger;
import java.security.SecureRandom;

public class DHAgreement {
   private DHPrivateKeyParameters key;
   private DHParameters dhParams;
   private BigInteger privateValue;
   private SecureRandom random;

   public void init(CipherParameters param) {
      AsymmetricKeyParameter param2;
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom param1;
         this.random = (param1 = (ParametersWithRandom)param).getRandom();
         param2 = (AsymmetricKeyParameter)param1.getParameters();
      } else {
         CipherParameters var10000 = param;
         SecureRandom param3;
         param3 = new SecureRandom.<init>();
         this.random = param3;
         param2 = (AsymmetricKeyParameter)var10000;
      }

      if (param2 instanceof DHPrivateKeyParameters) {
         DHAgreement var2 = this;
         DHPrivateKeyParameters var3;
         this.key = var3 = (DHPrivateKeyParameters)param2;
         var2.dhParams = var3.getParameters();
      } else {
         throw new IllegalArgumentException("DHEngine expects DHPrivateKeyParameters");
      }
   }

   public BigInteger calculateMessage() {
      int var1 = this.dhParams.getP().bitLength() - 1;
      this.privateValue = (new BigInteger(var1, this.random)).setBit(var1 - 1);
      return this.dhParams.getG().modPow(this.privateValue, this.dhParams.getP());
   }

   public BigInteger calculateAgreement(DHPublicKeyParameters pub, BigInteger message) {
      if (pub.getParameters().equals(this.dhParams)) {
         return message.modPow(this.key.getX(), this.dhParams.getP()).multiply(pub.getY().modPow(this.privateValue, this.dhParams.getP())).mod(this.dhParams.getP());
      } else {
         throw new IllegalArgumentException("Diffie-Hellman public key has wrong parameters.");
      }
   }
}
